//PROFILE-NO
{$WARNINGS OFF }

{$mode objfpc}{$H+}
unit Profinl;

interface

USES
  Dialogs,
  LCLType,
  LCLIntf;
CONST
{$IFDEF CPU64 }
  ProDLLMeas = 'profmeas64.so';
{$ELSE }
  ProDLLMeas = 'profmeas.so';
{$ENDIF }

PROCEDURE ProfStop; external ProDLLMeas;
PROCEDURE ProfEnter( mptr : Pointer; prozNrX : LongWord); external ProDLLMeas;
PROCEDURE ProfExit(prozNrX : LongWord); external ProDLLMeas;

PROCEDURE ProfActivate; external ProDLLMeas;
PROCEDURE ProfDeActivate; external ProDLLMeas;

PROCEDURE StopCounting; external ProDLLMeas;
PROCEDURE ContinueCounting; external ProDLLMeas;
PROCEDURE ProfSetComment ( comm   : String ); external ProDLLMeas;
PROCEDURE ProfAppendResults( progEnd : Boolean ); external ProDLLMeas;
FUNCTION  ProfIstAktiv : Boolean; external ProDLLMeas;
PROCEDURE ProfResultatLoeschen; external ProDLLMeas;

// Delphi-Functions that set process idle
PROCEDURE ShowMessage ( CONST Msg  : String );

PROCEDURE ShowMessageFmt(const Msg : string; Params : array of const );

function  MessageDlg( const Msg : String;         AType   : TMsgDlgType;
                      AButtons  : TMsgDlgButtons; HelpCtx : Longint ) : Integer;

function  MessageDlgPos( const Msg : string;         DlgType : TMsgDlgType;
                         Buttons   : TMsgDlgButtons; HelpCtx : Longint;
                         X, Y      : Integer) : Integer;

function MessageDlgPosHelp( const Msg : string;         DlgType : TMsgDlgType;
                            Buttons   : TMsgDlgButtons; HelpCtx : Longint;
                            X, Y      : Integer;        const HelpFileName : string)
                           : Integer;


// Delphi-TApplication-Functions that set process idle
VAR
  ProcessMessages : Procedure Of Object;
  HandleMessage : Procedure Of Object;

FUNCTION  AMessageBox( Text, Caption   : PChar;
                       Flags : Longint): Integer;

// Functions that set process idle
{$IFDEF FPC }
FUNCTION  MessageBox ( hWnd : HWND; lpText, lpCaption: PChar; uType : UINT ) : Integer;
{$ENDIF }

PROCEDURE Sleep   (zeit : DWORD );

TYPE
  TProfIntApp = Class
    PROCEDURE ProcessMessages;
    PROCEDURE HandleMessage;
  end;

VAR
  ProfintApplication : TProfIntApp;


IMPLEMENTATION
USES
  Forms,
  SysUtils;

TYPE
  TMessageStyle = (smsInformation, smsWarning, smsCritical);

CONST
  MBOK            = 1;
  MBYESNO         = 2;
  MBYESNOCANCEL   = 3;

  AnsOK           = 1;
  AnsNO           = 2;
  AnsYes          = 3;
  AnsCancel       = 4;
  AnsIgnore       = 5;

FUNCTION MBox(CONST Botschaft : String; Knoepfe : Word; style : TMessageStyle ) : Integer;
{$IFDEF VISIBLE }
VAR
  IntKnoepfe : LongInt;
  ergebnis   : Integer;
  BC         : Array[0..255] OF Char;
{$ENDIF }
BEGIN

{$IFDEF VISIBLE }
  StrPCopy(BC, Botschaft);
  CASE Knoepfe OF
    MBOK            : IntKnoepfe := MB_OK;
    MBYESNO         : IntKnoepfe := MB_YESNO;
    MBYESNOCANCEL   : IntKnoepfe := MB_YESNOCANCEL;
    ELSE
      IntKnoepfe := MB_RETRYCANCEL;
  END;
  CASE style OF
    smsWarning     : IntKnoepfe := IntKnoepfe OR MB_ICONWARNING;
    smsCritical    : IntKnoepfe := intKnoepfe OR MB_ICONSTOP;
    ELSE
      ;
  END;
{$IFDEF CPU64 }
  ergebnis := Application.MessageBox(BC,'CLProfiler64',IntKnoepfe);
{$ELSE }
  ergebnis := Application.MessageBox(BC,'CLProfiler',IntKnoepfe);
{$ENDIF }
  CASE ergebnis OF
    IDOK     : Result := AnsOK;
    IDNo     : Result := AnsNO;
    IDYes    : Result := AnsYES;
    IDCancel : Result := AnsCancel;
    ELSE
      Result := AnsIgnore;
  END;
{$ELSE }
{$IFDEF CPU64 }
  WriteLn('CLProfiler64: ' + Botschaft );
{$ELSE }
  WriteLn('CLProfiler: ' + Botschaft );
{$ENDIF }
  Result := AnsIgnore;
{$ENDIF }
END;

{$ASMMODE intel}
FUNCTION RDTSC : Int64;
BEGIN
ASM
  RDTSC;
END;
END;

// Check if CPU is intel-Compatible
PROCEDURE PruefeKompatibilitaet;
BEGIN
  Try
    RDTSC;
  Except
    MBox('This CPU is not Intel-Compatible', MB_OK, smsInformation);
    halt(0);
  End;
END;

PROCEDURE ShowMessage ( CONST Msg  : String );
BEGIN
  StopCounting;
  Dialogs.ShowMessage(Msg);
  ContinueCounting;
END;

PROCEDURE ShowMessageFmt(const Msg : string; Params : array of const );
BEGIN
  StopCounting;
  Dialogs.ShowMessageFmt(Msg, Params);
  ContinueCounting;
END;
{$IFDEF FPC }
FUNCTION MessageBox ( hWnd : HWND; lpText, lpCaption: PChar; uType : UINT ) : Integer;
BEGIN
  StopCounting;
  Result := LCLIntF.MessageBox(hWnd, lpText, lpCaption, uType);
  ContinueCounting;
END;
{$ENDIF }
FUNCTION AMessageBox( Text, Caption     : PChar;
                      Flags : LongInt ) : Integer;
BEGIN
  StopCounting;
  Result := Application.MessageBox(Text, Caption,Flags);
  ContinueCounting;
END;

PROCEDURE TProfIntApp.HandleMessage;
BEGIN
  ProfStop; Try; ProfEnter(NIL,1);
  Application.HandleMessage;
  finally; ProfExit(1); end;
END;

PROCEDURE TProfIntApp.ProcessMessages;
BEGIN
  ProfStop; Try; ProfEnter(NIL,1);
  Application.ProcessMessages;
  finally; ProfExit(1); end;
END;

FUNCTION MessageDlg( const Msg : String;         AType   : TMsgDlgType;
                     AButtons  : TMsgDlgButtons; HelpCtx : Longint ) : Integer;
BEGIN
  StopCounting;
  Result := Dialogs.MessageDlg(Msg,
                               AType,
                               AButtons,
                               HelpCtx);
  ContinueCounting;
END;

FUNCTION MessageDlgPos( const Msg : string;         DlgType : TMsgDlgType;
                        Buttons   : TMsgDlgButtons; HelpCtx: Longint;
                        X, Y      : Integer) : Integer;
BEGIN
  StopCounting;
  Result := Dialogs.MessageDlgPos(Msg,
                                  DlgType,
                                  Buttons,
                                  HelpCtx,
                                  X, Y);
  ContinueCounting;
END;

function MessageDlgPosHelp( const Msg : string;         DlgType      : TMsgDlgType;
                            Buttons   : TMsgDlgButtons; HelpCtx      : Longint;
                            X, Y      : Integer;  const HelpFileName : string)
                           : Integer;
BEGIN
  StopCounting;
  Result := Dialogs.MessageDlgPosHelp(Msg,
                                      DlgType,
                                      Buttons,
                                      HelpCtx,
                                      X, Y,
                                      HelpFileName);
  ContinueCounting;
END;

PROCEDURE Sleep( zeit : DWORD );
BEGIN
  StopCounting;
  SysUtils.Sleep(zeit);
  ContinueCounting;
END;

INITIALIZATION
  PruefeKompatibilitaet;
  ProfSetComment('None');
  ProfintApplication := TProfIntApp.Create;
  ProcessMessages := @ProfintApplication.ProcessMessages;
  HandleMessage   := @ProfintApplication.HandleMessage;
FINALIZATION
  ProfSetComment('At finishing application');
  ProfAppendResults(TRUE);
  ProfintApplication.Free;
end.

